from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class ApiMigration(MigrationABC):
    name = '0.3_ApiMigration'

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        MigrationABC.__init__(self)
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, 'Running upgrade')

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `AuthUsers` (
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `FirstName` VARCHAR(255),
                `LastName` VARCHAR(255),
                `EMail` VARCHAR(255),
                `Password` VARCHAR(255),
                `PasswordSalt` VARCHAR(255),
                `RefreshToken` VARCHAR(255),
                `ConfirmationId` VARCHAR(255) DEFAULT NULL,
                `ForgotPasswordId` VARCHAR(255) DEFAULT NULL,
                `OAuthId` VARCHAR(255) DEFAULT NULL,
                `RefreshTokenExpiryTime` DATETIME(6) NOT NULL,
                `AuthRole` INT NOT NULL DEFAULT '0',
                `CreatedAt` DATETIME(6) NOT NULL,
                `LastModifiedAt` DATETIME(6) NOT NULL,
                PRIMARY KEY(`Id`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `AuthUserUsersRelations`(
                `Id` BIGINT NOT NULL AUTO_INCREMENT,
                `AuthUserId` BIGINT DEFAULT NULL,
                `UserId` BIGINT DEFAULT NULL,
                `CreatedAt` DATETIME(6) NOT NULL,
                `LastModifiedAt` DATETIME(6) NOT NULL,
                PRIMARY KEY(`Id`),
                FOREIGN KEY (`AuthUserId`) REFERENCES `AuthUsers`(`Id`),
                FOREIGN KEY (`UserId`) REFERENCES `Users`(`UserId`)
            );
            """)
        )

    def downgrade(self):
        self._cursor.execute('DROP TABLE `AuthUsers`;')
        self._cursor.execute('DROP TABLE `AuthUserUsersRelations`;')

